function [fls] = compute_power_spectrum_res(fls)
fls = get_spectral_images(fls);

res_x = fls.exp.pixel_size_x;
res_y = fls.exp.pixel_size_y;
if res_x ~= res_y
    disp('Pixel size for X axis differes for pixel size for Y axis')
    disp('Let us assume the worst value')
    res = max([res_x res_y]);
else
    res = res_x;
end

for i = 1:length(fls.fluolines)
    [N, M] = size(fls.fluolines(i).image);
    array_ft = fftshift(fft2(fls.fluolines(i).image));
    array_ps = (abs(array_ft)/(N * M)).^2;
    
    %% Pad 2D power spectrum array to square shape with NaN values
    dim_difference = abs(N-M);
    dim_max = max(N,M);
    if N > M
        if ~mod(dim_difference,2)
            array_ps = [NaN(N,dim_difference/2) array_ps NaN(N,dim_difference/2)];
        else
            array_ps = [NaN(N,floor(dim_difference/2)) array_ps NaN(N,floor(dim_difference/2)+1)];
        end
    elseif N < M
        if ~mod(dim_difference,2)
            array_ps = [NaN(dim_difference/2,M); array_ps; NaN(dim_difference/2,M)];
        else
            array_ps = [NaN(floor(dim_difference/2),M); array_ps; NaN(floor(dim_difference/2)+1,M)];
        end
    end
    
    %% Compute radially average power spectrum
    [X, Y] = meshgrid( -dim_max/2:dim_max/2-1, -dim_max/2:dim_max/2-1 );
    dim_half = floor(dim_max/2) + 1;
    
    [theta, rho] = cart2pol(X, Y);
    rho = round(rho);
    ii = cell(dim_half, 1);
    
    for r = 1:dim_half
        ii{r} = find(rho == r-1);
    end
    
    for r = 1:dim_half
        fls.fluolines(i).ps_resolution(2, r) = nanmean( array_ps( ii{r} ) );
    end
    image_size = max([N M]) * res;
    fls.fluolines(i).ps_resolution(1, :) = linspace(1, dim_half, dim_half) / image_size;
end