%%% A template to plot fluorescence data %%%
%%% mcs_mesh.m could be upgraded to do this job, but here is a script in
%%% the meantime
%%% Feb 24, 2016

%%% In order to use datread(filename) we need in the header at least this
% # channels 8192
% # dim1 4
% # dim2 1
% # number-of-exposures 1
% # end-of-header
% 
% Would also be nice to see the MCS data and match the format approximately 

% clear
import io.*
import beamline.read_omny_pos

addpath ptycho
scan = [463:524];%[463:524]; %532:538

%%% Parameters of scan and detector %%%
num_channels = 1024;
% num_channels = 8192;
% For plotting in 2D, set = 1 to ignore %
numlines = 1;
%%% Parameters %%%
omnyposfile = ['~/Data10/specES1/scan_positions/scan_%05d.dat']; % Set to empty to ignore and do a mesh, e.g. 
scanning_points = [];   % Example = '+b'
upsampling = 2;         % Ratio of scanning vs plotting points in one dimensions. If (= 1) data points and meshpoints are approx equal

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Choose energies to plot in keV %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
energy_to_plot = 6.49;  % Mn Kb1 edge
energy_label = 'Mn Kb1';

% energy_to_plot = 6.915;  % Co Ka2 edge
% energy_label = 'Co Ka2';

% energy_to_plot = 8.26;  %
% energy_label = 'Ni Kb1';

energy_peak_width = 0.5;      % Specify the FWHM of the integration range

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Choose energies to plot in colors in keV %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

energy_to_plot_r = 6.49;  % Mn Kb1 edge
label_r = 'Mn Kb1';
energy_peak_width_r = 0.5; % Specify the FWHM of the integration range

energy_to_plot_g = 8.26;  % 
label_g = 'Ni Kb1';
energy_peak_width_g = 0.5; % Specify the FWHM of the integration range

energy_to_plot_b = [];%6.915;  % Mn Kb1 edge
label_b = 'Co Ka2';
energy_peak_width_b = 0.5; % Specify the FWHM of the integration range

saturation = 1.2;   % Saturate colorbar to see colors better

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Get spectrum of a ROI %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
roi_spectrum = [];
roi_spectrum{1} = [8 12 -7 -5]; %[xi xf yi yf] Define the ROI, set empty = [] to ignore
roi_spectrum{2} = [16 20 -7 -5]; %[xi xf yi yf] Define the ROI
% roi_spectrum{3} = [23 23.5 -4.5 -4]; %[xi xf yi yf] Define the ROI


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Calibration of spectrum %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
spectrum_step = 8.905*(8192/num_channels)/5656;
energy_axis = [0:spectrum_step:(num_channels-1)*spectrum_step]; % Empty to see the pixels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

index_center = find(abs(energy_axis-energy_to_plot)==min(abs(energy_axis-energy_to_plot)),1)-1;
index_start  = find(abs(energy_axis-energy_to_plot+energy_peak_width/2)==min(abs(energy_axis-energy_to_plot+energy_peak_width/2)),1)-1;
index_end    = find(abs(energy_axis-energy_to_plot-energy_peak_width/2)==min(abs(energy_axis-energy_to_plot-energy_peak_width/2)),1)-1;
spectrum_range  = [index_start:index_end]; 

%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% RGB images %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(energy_to_plot_r)
    index_center_r = find(abs(energy_axis-energy_to_plot_r)==min(abs(energy_axis-energy_to_plot_r)),1)-1;
    index_start_r  = find(abs(energy_axis-energy_to_plot_r+energy_peak_width_r/2)==min(abs(energy_axis-energy_to_plot_r+energy_peak_width_r/2)),1)-1;
    index_end_r    = find(abs(energy_axis-energy_to_plot_r-energy_peak_width_r/2)==min(abs(energy_axis-energy_to_plot_r-energy_peak_width_r/2)),1)-1;
    
    spectrum_range_r = [index_start_r:index_end_r];
else
    spectrum_range_r = [];
end

if ~isempty(energy_to_plot_g)
    index_center_g = find(abs(energy_axis-energy_to_plot_g)==min(abs(energy_axis-energy_to_plot_g)),1)-1;
    index_start_g  = find(abs(energy_axis-energy_to_plot_g+energy_peak_width_g/2)==min(abs(energy_axis-energy_to_plot_g+energy_peak_width_g/2)),1)-1;
    index_end_g    = find(abs(energy_axis-energy_to_plot_g-energy_peak_width_g/2)==min(abs(energy_axis-energy_to_plot_g-energy_peak_width_g/2)),1)-1;
    
    spectrum_range_g = [index_start_g:index_end_g];
else
    spectrum_range_g = [];
end

if ~isempty(energy_to_plot_b)
    index_center_b = find(abs(energy_axis-energy_to_plot_b)==min(abs(energy_axis-energy_to_plot_b)),1)-1;
    index_start_b  = find(abs(energy_axis-energy_to_plot_b+energy_peak_width_b/2)==min(abs(energy_axis-energy_to_plot_b+energy_peak_width_b/2)),1)-1;
    index_end_b    = find(abs(energy_axis-energy_to_plot_b-energy_peak_width_b/2)==min(abs(energy_axis-energy_to_plot_b-energy_peak_width_b/2)),1)-1;
    
    spectrum_range_b = [index_start_b:index_end_b];
else
    spectrum_range_b = [];
end


% 5109 -> Cu k-alpha  8.047 (by eye)
% 5656 -> Cu k-beta   8.905

imwrite_folder = sprintf('~/Data10/analysis/online/fluorescence/');

loaddata = 1;
if exist('scans_old','var')
    if (numel(scans_old)==numel(scan))
        if all(scans_old == scan)
            loaddata = 0;
        end
    end
end

%
numscans = numel(scan);

if loaddata
    data = [];
    for ii = 1:numscans
        filename{ii} = fullfile(sprintf('~/Data10/x123/gccDppConsoleLinux/data/x123_%05d.dat',scan(ii)));
        data_str{ii} = image_read(filename{ii});
        data = [data data_str{ii}.data];
    end
else
    display('Using data already loaded')
end

% fid = fopen(filename,'r');
% data = fscanf(fid,'%d');
% fclose(fid);

% numels = numel(data)/num_channels;
% data = reshape(data,[num_channels numels]);

title_str = sprintf('Scan: S%05d - S%05d',scan(1),scan(end));

if numlines>1
    if mod(numels,numlines) ~= 0
        error('The number of lines is not consistent with the number of elements in the measurement')
    end
    if isempty(spectrum_range)
        img_sp = sum(data,1);
    else
        img_sp = sum(data(spectrum_range,:),1);
    end
    img_sp = reshape(img_sp,[numlines numels/numlines]);
    figure(3)
    imagesc(img_sp)
    colormap bone
    colorbar
    axis xy equal tight
    title(['Scanning image: ' title_str],'interpreter','none')
end





%%%%%%%%%%%%%%%%%%%%%
%%% Plot spectrum %%%
%%%%%%%%%%%%%%%%%%%%%

sumdata = sum(data,2);

figure(1);
clf
if ~isempty(energy_axis)
    semilogy(energy_axis,sum(data,2))
    xlabel('keV')
end
title(['Total spectrum ' title_str])
legendarray = [];
legendtext = {};
labelcolortext = [];

hold on
if ~isempty(spectrum_range)
    h1 = semilogy([min(spectrum_range) min(spectrum_range)]*spectrum_step,[1 max(sum(data,2))],'k');
    semilogy([max(spectrum_range) max(spectrum_range)]*spectrum_step,[1 max(sum(data,2))],'k');
    legendarray = h1;
    legendtext{end+1} = 'Image';
end
if ~isempty(energy_to_plot_r)
    hr = semilogy([min(spectrum_range_r) min(spectrum_range_r)]*spectrum_step,[1 max(sum(data,2))],'--r');
    semilogy([max(spectrum_range_r) max(spectrum_range_r)]*spectrum_step,[1 max(sum(data,2))],'--r');
    legendarray = [legendarray hr];
    legendtext{end+1} = 'Red';
    labelcolortext = [labelcolortext 'Red:' label_r ', '];
end
if ~isempty(energy_to_plot_g)
    hg = semilogy([min(spectrum_range_g) min(spectrum_range_g)]*spectrum_step,[1 max(sum(data,2))],'--g');
    semilogy([max(spectrum_range_g) max(spectrum_range_g)]*spectrum_step,[1 max(sum(data,2))],'--g');
    legendarray = [legendarray hg];
    legendtext{end+1} = 'Green';
    labelcolortext = [labelcolortext 'Green:' label_g ', '];
end
if ~isempty(energy_to_plot_b)
    hb = semilogy([min(spectrum_range_b) min(spectrum_range_b)]*spectrum_step,[1 max(sum(data,2))],'--b');
    semilogy([max(spectrum_range_b) max(spectrum_range_b)]*spectrum_step,[1 max(sum(data,2))],'--b');
    legendarray = [legendarray hb];
    legendtext{end+1} = 'Blue';
    labelcolortext = [labelcolortext 'Blue:' label_b ', '];
end
    
hold off
legend(legendarray,legendtext)


% figure(2);
% semilogy(sumdata)
% title(['Sum of all spectra ' title_str],'interpreter','none')
% % legend

if ~isempty(omnyposfile)
    x = [];
    y = [];
    for ii = 1:numscans
        posdata{ii} = read_omny_pos(sprintf(omnyposfile,scan(ii)));
        x = [x; -posdata{ii}.Average_x_st_fzp];
        y = [y; -posdata{ii}.Average_y_st_fzp];
    end
    
    if isempty(spectrum_range)
        img_sp = sum(data,1).';
    else
        img_sp = sum(data(spectrum_range,:),1).';
    end
    
    %%% Interpolate %%%
    Nx = upsampling*round(sqrt( (max(x)-min(x))*numel(x)/(max(y)-min(y))));
    Ny = upsampling^2*round(numel(x)/Nx);
    F = scatteredInterpolant(x,y,img_sp);
    xg = linspace(min(x),max(x),Nx);
    yg = linspace(min(y),max(y),Ny);
    [Xg,Yg] = meshgrid(xg,yg);
    img_sp_interp = F(Xg,Yg);
    %%% Plot
    figure(3)
    imagesc(xg,yg,img_sp_interp);
    colormap bone
    colorbar
    axis xy equal tight
    title(['Image ' title_str],'interpreter','none')
    if ~isempty(scanning_points)
        hold on
        plot(x,y,scanning_points)
        hold off
    end
    xlabel([energy_label ' ' num2str(energy_to_plot) ' keV'])

    if (~isempty(spectrum_range_r))||(~isempty(spectrum_range_g))||(~isempty(spectrum_range_b))
        img_sp_r = sum(data(spectrum_range_r,:),1).';
        img_sp_g = sum(data(spectrum_range_g,:),1).';
        img_sp_b = sum(data(spectrum_range_b,:),1).';
        F_r = scatteredInterpolant(x,y,img_sp_r);
        F_g = scatteredInterpolant(x,y,img_sp_g);
        F_b = scatteredInterpolant(x,y,img_sp_b);
        img_sp_interp_r = F_r(Xg,Yg);
        img_sp_interp_g = F_g(Xg,Yg);
        img_sp_interp_b = F_b(Xg,Yg);
        img_color = [];
        img_color(:,:,1) = img_sp_interp_r;
        img_color(:,:,2) = img_sp_interp_g;
        img_color(:,:,3) = img_sp_interp_b;
        img_color = img_color/max(img_color(:));
        if saturation > 1
            img_color = img_color*saturation;
            img_color(img_color>1) = 1;
        end
        
        if ~exist(imwrite_folder,'dir')
            try
                mkdir(imwrite_folder)
            catch
                warning(['Could not create folder ' imwrite_folder])
            end
        end

        figure(4)
        image(img_color);
        axis xy equal tight
        set(gca,'XTick',[],'YTick',[])
        xlabel(labelcolortext)
        title(title_str,'interpreter','none')
        savename = sprintf('S%05d_S%05d_colorful_image',scan(1),scan(end));
        imgfile = fullfile(imwrite_folder,[savename '.jpg']);
        display(['Saving ' imgfile])
        print('-djpeg','-r100',imgfile)
        img_transp = sum(img_color,3)*1/3;
        imgfile = fullfile(imwrite_folder,[savename '.png']);
        display(['Saving ' imgfile])
        imwrite(flipud(img_color),imgfile,'Alpha',flipud(img_transp))
    end
    
    %%% Get spectrum of ROI %%%
    spectrum_roi = [];
    
    if ~isempty(roi_spectrum)
        figure(6)
        clf
        for ii = 1:numel(roi_spectrum)
            xi = roi_spectrum{ii}(1);
            xf = roi_spectrum{ii}(2);
            yi = roi_spectrum{ii}(3);
            yf = roi_spectrum{ii}(4);
            spectrum_roi(:,ii) = sum(data(:, (x>xi)&(x<xf)&(y>yi)&(y<yf) ),2);
            
        end
        hsp = semilogy(energy_axis',spectrum_roi);
        title(['ROI spectrum ' title_str])
        figure(3)
        hold on
        for ii = 1:numel(roi_spectrum)
            rectangle('Position',[roi_spectrum{ii}(1) roi_spectrum{ii}(3)...
                roi_spectrum{ii}(2)-roi_spectrum{ii}(1) roi_spectrum{ii}(4)-roi_spectrum{ii}(3)],'edgecolor',hsp(ii).Color)
        end
        hold off
    end
end



scans_old = scan;














%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing language this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS software package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.