#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an test functions to process stxm of a hdf5 dataset.
'''
if __name__ == '__main__':
  #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import matplotlib as mpl
#if __name__ == '__main__':
#  mpl.use('WXAgg')
  #or mpl.use('WX')
  #matplotlib.get_backend()

import sys,os,logging,re
import json, zmq, h5py
import multiprocessing as mproc
import numpy as np
import pylab as plt #used for the colormaps
from scipy import ndimage as ndi
from Clamp import Clamp,ClampIn,ClampOut,ClampLst,MultiProc,SingleProc
import utilities as ut
from libDetXR import LibDetXR
import procMoment as pm
LibDetXR()

_log=logging.getLogger(__name__)

def LogOut(s):
  log=logging.root.handlers[0].stream
  log.write(s)
  log.flush()

class ProcSTXM(pm.ProcMoment):
  def __init__(self,args):
    self.args=args
    if args.dst==None:
      assert(args.src[0]=='hdf')
      pt,fn=os.path.split(args.src[1])
      assert(fn.startswith('scan_'))
      args.dst=('hdf',os.path.join(pt,'moments_'+fn[5:]),'entry/data/moments')
    if args.matMsk==None and args.src[0]=='hdf':
      pt=os.path.dirname(args.src[1])
      args.matMsk=os.path.join(pt,'analysis/data/pilatus_valid_mask.mat')

    if args.matMsk!=None:
      try:
        self.SetMskMat(args.matMsk,args.matMskFlt,args.verbose&8)
        del self.mskIdx #turn of masking. CHECK IF MASK NEEDED!!!
      except IOError:
        _log.warning('could not load valid mask: '+args.matMsk)
    if args.roi!=None:
      self.roi=(slice(args.roi[1],args.roi[1]+args.roi[3]),slice(args.roi[0],args.roi[0]+args.roi[2]))
      self.shape=(args.roi[3],args.roi[2])
      self.yy=np.arange(self.shape[0],dtype=np.uint32)
      self.xx=np.arange(self.shape[1],dtype=np.uint32)

    dtype=np.float32
    w=3 #t,x,y
    self.resArr=np.ndarray((w,),dtype=dtype)
    self.resArr[:]=0

  def Close(self):pass

  def Process(self,idx,data):
    try:
      data.ravel()[self.mskIdx]=0
    except AttributeError: pass
    try:
      data=data[self.roi]
    except AttributeError:
      center=self.FindCenter(data)
      roiSize=self.args.roiSize
      sh=data.shape
      rMax=min(roiSize[1],center[1],sh[1]-center[1],center[0],sh[0]-center[0])
      self.roi=roi=(slice(center[1]-rMax,center[1]+rMax),
                    slice(center[0]-rMax,center[0]+rMax))

      raw=data
      data=data[roi]
      self.yy=np.arange(data.shape[0],dtype=np.uint32)
      self.xx=np.arange(data.shape[1],dtype=np.uint32)
      if self.args.verbose&8:
        print center,roiSize,roi
        plt.figure(2)
        plt.subplot(211)
        plt.imshow(np.log(raw+1))
        plt.subplot(212)
        plt.imshow(np.log(data+1))
        plt.show(block=False)

    sx=data.sum(0)
    sy=data.sum(1)
    x=(self.xx*sx).sum()/np.float(sx.sum())
    y=(self.yy*sy).sum()/np.float(sy.sum())
    t=sx.sum()
    self.resArr[0]=t
    self.resArr[1]=x
    self.resArr[2]=y
    return self.resArr


  def FindCenter(self,arr,show=False):
    m=ndi.median_filter(arr, 5)
    sx=m.sum(0)
    sy=m.sum(1)
    shape=arr.shape
    xx=np.arange(shape[1])
    yy=np.arange(shape[0])
    x=(xx*sx).sum()/sx.sum()
    y=(yy*sy).sum()/sy.sum()
    if show:
      print x,y
      plt.figure(1)
      plt.subplot(211)
      plt.plot(sx)
      plt.subplot(212)
      plt.plot(sy)
      plt.show(block=False)
    center=(int(x+.5),int(y+.5))
    return center

class ShowSTXM():
  def SetHDFSrc(self,hdfFile,hdfEntry,width,hdfRawFile=None,hdfRawEntry=None):
    #base='$IN_DIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/'
    #fid = h5py.h5f.open(os.path.join(base,'moments_00106-00161.hdf5'))
    #hid = h5py.h5o.open(fid,'entry/data/moments/c')
    fid = h5py.h5f.open(hdfFile,flags=h5py.h5f.ACC_RDONLY)
    hid = h5py.h5o.open(fid,hdfEntry)
    data=h5py.Dataset(hid).value
    self.data=data.reshape((width,-1,3))
    if hdfRawFile!=None:
      fid = h5py.h5f.open(hdfRawFile,flags=h5py.h5f.ACC_RDONLY)
      self.hidRaw = h5py.h5o.open(fid,hdfRawEntry)

  def Show(self):
    data=self.data

    fig=plt.figure()#(toolbar=False)
    ax=plt.subplot(221)
    #plt.imshow(data[...,1],interpolation='nearest')
    plt.imshow(data[...,1],interpolation='nearest',cmap=mpl.cm.gray)
    plt.colorbar()
    ax.invert_xaxis()
    ax.set_title('DPC x')
    ax.format_coord=self.OnFmtCoord

    ax=plt.subplot(222)
    #plt.imshow(data[...,2],interpolation='nearest')
    plt.imshow(data[...,2],interpolation='nearest',cmap=mpl.cm.gray)
    plt.colorbar()
    ax.invert_xaxis()
    ax.set_title('DPC y')
    ax.format_coord=self.OnFmtCoord

    ax=plt.subplot(223)
    plt.imshow(data[...,0],interpolation='nearest',cmap=mpl.cm.gray)
    plt.colorbar()
    ax.set_title('transmission')
    ax.invert_xaxis()
    ax.format_coord=self.OnFmtCoord

    rngdx=(data[...,1].min(),data[...,1].max(),data[...,1].mean())
    rngdy=(data[...,2].min(),data[...,2].max(),data[...,2].mean())

    sldX0 = mpl.widgets.Slider(fig.add_axes([0.60, 0.40, 0.3, 0.03], ), 'dx0', rngdx[0], rngdx[1], valinit=rngdx[2])
    sldY0 = mpl.widgets.Slider(fig.add_axes([0.60, 0.35, 0.3, 0.03], ), 'dy0', rngdy[0], rngdy[1], valinit=rngdy[2])
    sldsG = mpl.widgets.Slider(fig.add_axes([0.60, 0.30, 0.3, 0.03], ), 'sGamma', 0.25, 2.0, valinit=1)
    sldT0 = mpl.widgets.Slider(fig.add_axes([0.60, 0.25, 0.3, 0.03], ), 't0', 0.0, 0.9, valinit=.25)
    sldTS = mpl.widgets.Slider(fig.add_axes([0.60, 0.20, 0.3, 0.03], ), 'tScale', 1., 10., valinit=1)
    sldTG = mpl.widgets.Slider(fig.add_axes([0.60, 0.15, 0.3, 0.03], ), 'tGamma', 0.25, 2.0, valinit=1)
    self.sldLst= (sldX0,sldY0,sldsG,sldT0,sldTS,sldTG)
    for sld in self.sldLst:
      sld.on_changed(self.OnUpdate)

    fig=plt.figure()
    ax=fig.gca()
    self.hsv=np.ndarray(data.shape,dtype=np.float32)
    self.tMin=data[...,0].min()
    self.tMax=data[...,0].max()

    v=map(lambda v:v.val, self.sldLst)
    hsv=self.NormHSV(*v)
    self.himgHSV=plt.imshow(mpl.colors.hsv_to_rgb(hsv),interpolation='nearest')
    ax.set_title('combined t,dx,dy')
    ax.invert_xaxis()
    ax.format_coord=self.OnFmtCoord
    cid = fig.canvas.mpl_connect('button_press_event', self.OnEvtBtnPress)
    cid = fig.canvas.mpl_connect('motion_notify_event', self.OnEvtMotion)

    cm=mpl.cm.hsv
    n=cm.N
    theta = np.linspace(0,2*np.pi,n);
    r = np.arange(2)
    X, Y = np.meshgrid(theta, r)

    C = np.ndarray(shape=(len(theta),len(r)))
    C[:,0]=np.linspace(0,1,n)
    C[:,1]=C[:,0]
    ax=fig.add_axes([0.90, 0.80, 0.1, 0.1],polar=True)
    ax.yaxis.set_major_locator(plt.NullLocator())
    plt.pcolormesh(X, Y, C.T,cmap=cm)

    try:
      hidRaw=self.hidRaw
    except AttributeError:pass
    else:
      fig=plt.figure()
      ax=fig.gca()
      data=h5py.Dataset(hidRaw)[100,...]
      avg=np.average(data); std=np.std(data)
      vmin=np.min(data);vmax=np.max(data)
      vmin=max(vmin,avg-3*std);vmax=min(vmax,avg+3*std)
      #vmax=500
      vmin=.1
      #self.rawImg = ax.imshow(data,interpolation='nearest',cmap=mpl.cm.jet, vmin=vmin, vmax=vmax)
      self.rawImg = ax.imshow(data,interpolation='nearest',cmap=mpl.cm.jet, norm=mpl.colors.LogNorm(vmin=1, vmax=vmax))
      ax.set_axis_bgcolor('black')
      colBar=fig.colorbar(self.rawImg,orientation='vertical')
      ax.invert_xaxis()
      ax.axis('image')

    plt.show()
    #tbar = f.get_navigation_toolbar()


  def OnFmtCoord(self,x, y):
    x=int(round(x));  y=int(round(y))
    try:
      tr=self.data[y,x,0]
      dx=self.data[y,x,1]
      dy=self.data[y,x,2]
      return 'x:%d y:%d idx:%d trans:%g dx:%g dy:%g'%(x,y,y*self.data.shape[1]+x,tr,dx,dy)
    except IndexError as e:
      return str(e)

  def NormHSV(self,dx0,dy0,sGamma,t0,tScale,tGamma):
    hsv=self.hsv
    data=self.data
    tMax=self.tMax
    tMin=self.tMin

    #n=np.prod(data.shape[0:2])
    #avg=data.reshape((n,3)).mean(axis=0)
    #std=data.reshape((n,3)).std(axis=0)

    #2: intensity
    hsv[...,2]=(data[...,0]-tMin)/(tMax-tMin)
    if tGamma!=1: hsv[...,2]=hsv[...,2]**tGamma #gamma correction
    #hsv[...,2]=t0+hsv[...,2]*(1.-t0)*tScale
    hsv[...,2]=t0+(1.-hsv[...,2])*(1.-t0)*tScale #invert transmission
    hsv[hsv[...,2]>1,2]=1. #limit to maximum value 1.

    xx=data[...,1]-dx0
    yy=data[...,2]-dy0
    #h=angle s=amplitude v=transmission
    #h=np.arctan2(xx,yy)/(2*np.pi)+.5
    h=np.arctan2(yy,xx)/(2*np.pi)+.5
    s=np.sqrt(xx**2+yy**2);s/=s.max()
    if sGamma!=1: s=s**sGamma #gamma correction

    hsv[...,0]=h
    hsv[...,1]=s
    return hsv

  def OnUpdate(self,val):
    #print val
    v=map(lambda v:v.val, self.sldLst)
    #print v
    hsv=self.NormHSV(*v)
    himgHSV=self.himgHSV
    himgHSV.set_array(mpl.colors.hsv_to_rgb(hsv))
    himgHSV.axes.figure.canvas.draw_idle()

  def OnEvtBtnPress(self,event):
    x=int(round(event.xdata)); y=int(round(event.ydata))
    v=self.data[y,x,:]
    if event.button==1:
      try:
        hidRaw=self.hidRaw
      except AttributeError:pass
      else:
        idx=y*self.data.shape[1]+x
        data=h5py.Dataset(hidRaw)[idx]
        img=self.rawImg
        ax=img.axes
        fig=ax.figure
        img.set_array(data)
        #self.rawFig=fig
        ax.set_title('image %d'%idx)
        #self.rawFig.show()
        fig.canvas.draw_idle()
    elif event.button==3:
      (sldX0,sldY0,sldsG,sldT0,sldTS,sldTG)=self.sldLst
      sldX0.set_val(v[1])
      sldY0.set_val(v[2])
    pass

  def OnEvtMotion(self,event):
    pass
    ax=event.inaxes
    if ax:
      x=int(round(event.xdata))
      y=int(round(event.ydata))
      try:
    #    #childLst=ax.get_children()
        im=ax.get_images()[0]
        v=self.data[y,x,:]
      except IndexError as e:
        pass
      else:
        (sldX0,sldY0,sldsG,sldT0,sldTS,sldTG)=self.sldLst
        #print x,y,v[1]-sldX0.val,v[2]-sldY0.val,v#,event.button

def RunWithArgs(args):
  if type(args)==dict:
    args=ut.dotdict(args)
    default={
            'numFrm':sys.maxint,
            'matRoi':None,
            'dst':None,
            'size':None,
            'roiSize':(0,128),
            'roiShape':'rect',
            'center':None,
            'proc':None,
            'matMsk':None,
            'matMskFlt':False,
            'multiproc':None,
            'cmpr':None,
            'verbose':3
            } #setup default values
    for k, v in default.items():
      if not args.has_key(k):
        args[k]=v
    if args.multiproc==True: args.multiproc=[3]
    _log.setLevel(logging.INFO if args.verbose&1 else logging.WARNING)
  else:
    args=ut.dotdict(args.__dict__)
  for clp in (args.src,args.dst):
    if clp[0] in ('zmq','zmqMsg'):
      clp[1]=ut.ZMQ.lst2dict(clp[1:])
      del clp[2:]

  if args.center:
    args.roi=[args.center[0]-args.roiSize[1],
              args.center[1]-args.roiSize[1],
              args.roiSize[1]*2,
              args.roiSize[1]*2]
  else:
    args.roi=None
  args.procmode='stxm'
  args.comment='transmission, x-deviation, y-deviation'

  #print '>>>>',args,'<<<<<'
  size=args.size
  if size==None:
    if args.src[0]=='hdf':
      try:
        fid=ut.OpenFile(args.src[1],'r')
        dsid=h5py.h5d.open(fid,'/entry/data/spec')
      except KeyError:
        try:
          gid=h5py.h5g.open(fid,'/entry/data')
        except KeyError as e:
          raise(e)
        else:
          numY=0
          for hidStr in h5py.h5g.GroupIter(gid):
            if re.match('^S\d*$',hidStr):
              numY+=1
          del gid #-> read  MultiProc doc
      else:
        numY=dsid.shape[0]
        del dsid #-> read  MultiProc doc
      dsid=h5py.h5d.open(fid,args.src[2])
      numX=h5py.Dataset(dsid).shape[0]/numY
      del dsid #-> read  MultiProc doc
      size=(numX,numY)
      ut.CloseFile(fid)
  if args.multiproc!=None:
    MultiProc(ClampIn,ProcSTXM,pm.ClampOutMoment,args)
  else:
    SingleProc(ClampIn,ProcSTXM,pm.ClampOutMoment,args)

  if args.dst[0]in('zmq','zmqMsg'):
    param=args.dst[1]
    if type(param)==str: param=json.loads(param, object_hook=ut.ConvUtf8)
    sock=ut.ZMQ.Socket(param,zmq.PUSH,4)
    sock.send_json({'cmd':'stop'})
    sock.close()
  elif args.dst[0]=='hdf':
    if args.verbose&16:
      try:
        shape=tuple(reversed(size))+(3,)
      except AttributeError:
        LogOut('could not determine result size')
      else:
        hdfFile=args.dst[1]
        hdfEntry=args.dst[2]
        width=size[1]
        if args.src[0]=='hdf':
          hdfRawFile=args.src[1]
          hdfRawEntry=args.src[2]
        else:
          hdfRawFile=hdfRawEntry=None
        ss=ShowSTXM()
        ss.SetHDFSrc(hdfFile,hdfEntry,width,hdfRawFile,hdfRawEntry)
        ss.Show()


if __name__ == '__main__':
  import os,sys,argparse #since python 2.7

  def ParseArgs():
    if len(sys.argv)>1 and sys.argv[1]=='show':
      hdfFile=sys.argv[2]
      hdfEntry=sys.argv[3]
      width=int(sys.argv[4])
      try:
        hdfRawFile=sys.argv[5]
        hdfRawEntry=sys.argv[6]
      except IndexError:
          hdfRawFile=hdfRawEntry=None
      ss=ShowSTXM()
      ss.SetHDFSrc(hdfFile,hdfEntry,width,hdfRawFile,hdfRawEntry)
      ss.Show()
      return

    (h,t)=os.path.split(sys.argv[0]);cmd='\n  '+(t if len(h)>3 else sys.argv[0])+' '
    exampleCmd=('--src hdf $IN_DIR/stxm_pinhole/scan_00158-00198.hdf5 entry/data/pilatus_1 -v',
                '--src hdf $IN_DIR/e14472/scan_00033.hdf5 /entry/data/pilatus_1',
                '--src hdf $IN_DIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/scan_00106-00161.hdf5 /entry/data/pilatus_1 --size 66 56 --multiproc',
                'show  $IN_DIR/e14472/STXM_00033.hdf5 entry/data/STXM 10',
                'show  $IN_DIR/e14472/STXM_00033.hdf5 entry/data/STXM 10 $IN_DIR/e14472/scan_00033.hdf5 /entry/data/pilatus_1',
                'show $IN_DIR/stxm_pinhole/STXM_00158-00198.hdf5 entry/data/STXM 41 $IN_DIR/stxm_pinhole/scan_00158-00198.hdf5 entry/data/pilatus_1')

    epilog='''\n
the verbose bits are:
   bit0(0x01): basic
   bit1(0x02): progress
   bit2(0x04):
   bit3(0x08): gui masks
   bit4(0x10): show final result
   bit5(0x20):
   bit6(0x40):
   bit6(0x80):
   bit6(0x100): multiprocessing

   default is 1+2=3 (no -v)
   -v is 1+2+8+16+256=283
   -v0 is turn all off

Data Source description:'''+ClampIn.__types__+'''
Data Destination description:
  HDF file from a given object:
    --src hdf FILE ELEM
    --src hdf myfile.hdf5 entry/mygrp/dataset
  ZMQ stream
    --src zmq JSON-config (with addr and optional queueSz, ifType and timeout)
    --src zmq '{"addr":"tcp://localhost:8080","queueSz":4,"ifType":"PULL"}',

to show result:'''+cmd+''' show HDF5FILE HDF5ENTRY WIDTH [RAWHDF5FILE RAWHDF5ENTRY]

Examples:
  IN_DIR=/scratch/detectorData'''+''.join(map(lambda s:cmd+s,exampleCmd))+'\n '

    roiSize=(0,128)
    roiShape='rect'
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog=epilog)

    parser.add_argument('--src',   nargs='+', required=True, help='data source (see below for examples)')
    parser.add_argument('--dst',  nargs='+', metavar=('mode','param'), required=True, help='data destination: hdf or zmq sink')
    parser.add_argument('--size',  nargs=2,   type=int, metavar=('NUMX','NUMY'), help='size of the STXM image')
    parser.add_argument('--roiSize', nargs=2, type=int, default=roiSize, metavar=('RMIN','RMAX'), help='size of region of interest used for data analysis around the center (default is %d to %d pixel for both directions)'%roiSize)
    parser.add_argument('--roiShape', choices=('rect','circ'), default=roiShape,help='shape of data analysis ROI (default %s)'%roiShape)
    parser.add_argument('--center',nargs=2, type=int, metavar=('CTRX','CTRY'),help='center of data analysis (if not set, it is automatically detected)')
    parser.add_argument('--proc',choices=('moment','phase'), help='processing mode')
    parser.add_argument('--matMsk', help='file with the valid pixel mask (!=0 for valid)')
    parser.add_argument('--matMskFlt', action='store_true', help='filter MATLAB Mask cols and rows')
    parser.add_argument('-v',          dest='verbose', type=int, nargs='?', const=283, default=3, help='verbose (optional level bitmask)')
    parser.add_argument('-m', '--multiproc', nargs='?', const=[3], default=None,help='use multiprocessing method topo,[numproc]')
    parser.add_argument('--cmpr',                                          help='compression method for hdf5 output (zlib[lvl],lz4,lzf,cbf)')

    #'Usage:\n')
    #'[trans,dpcx,dpcy,df]=%s(<(first) scan number>, <no. of scan lines> [[,<name>,<value>] ...]);\n',mfilename);
    #'The optional <name>,<value> pairs are:\n');
    #'''DetectorNumber'',<1-Pilatus 2M, 2-Pilatus 300k, 3-Pilatus 100k>\n');
    #'''Nx'',<no. of points per line>         default is %d (0 means automatic determination from first scan line)\n',Nx);
    #'''ROIdim'',<no. of points>              region of interest used for data analysis, default is %d\n',roi_dim);
    #'''CenX'',<point>                        0 means automatic determination, default is %d\n',cen_x);
    #'''CenY'',<point>                        0 means automatic determination, default is %d\n',cen_y);
    #'''DarkFieldR'',<min. radius>            dark field integration starts at this radius, default is %.0f\n',dark_field_r);
    #'''FigNo'',<integer value>               figure number for data display, default is %d\n',fig_no);
    #'''DirPerLine'',<0-no,1-yes>             separate directory for each scan line, default is %d\n',dir_per_line);
    #'''ZeroOrderR'', <min. radius>           pixel values inside this radius are set to zero, default is %d\n', zero_order_r);
    #'''FirstMoment'',<0-no,1-yes>            calculate the first moment rather than a Fourier transform to get the differential phase contrast, default is %d\n',first_moment);
    #'''IntegratedPhase'',<0-no,1-yes>        differentiate additionally the sum signal and re-differentiate it, default is %d\n',integrated_phase);
    #'''UpdateInterval'',<integer N>          update the plot each Nth line, default is %d\n',update_interval);
    #'''FigureDir'',''directory''               save the resulting plot in eps, jpeg and Matlab fig format, '''' for no saving, default is %s\n',figure_dir);
    #'''DataDir'',''directory''                 save the resulting data as Matlab file, '''' for no saving, default is %s\n',data_dir);
    #'''FilenameValidMask'',<path and filename> Matlab file with the valid pixel indices ind_valid, [] for no valid pixel mask,\n');
    #'                                          default is %s\n',filename_valid_mask);
    #'Additional <name>,<value> pairs recognized by compile_x12sa_filename and by image_read can be specified. Please call them for an overview\n');
    #'\n');
    #'At least the (first) scan number and the number of scan lines have to be specified as input parameter.');

    args = parser.parse_args()
    if type(args.multiproc)==str:
      args.multiproc=map(int,args.multiproc.split(','))
      assert(len(args.multiproc)<=2)
    ut.LogConfig(args, {__name__     : -1, 'utilities'  : -1,})
    if args.dst==None:
      assert(args.src[0]=='hdf')
      pt,fn=os.path.split(args.src[1])
      assert(fn.startswith('scan_'))
      args.dst=('hdf',os.path.join(pt,'STXM_'+fn[5:]),'entry/data/STXM')
    RunWithArgs(args)

#------------------ Main Code ----------------------------------
  ParseArgs()
