#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
Processing Sample:

this shows a simple example of processing jobs using clamp classes and Single/MultiProcessing
topology.
'''

import logging, sys, os, glob, stat, time, json
import numpy as np
import utilities as ut
from Clamp import Clamp,ClampIn,ClampOut,ClampLst,MultiProc,SingleProc
import multiprocessing as mproc

_log=logging.getLogger(__name__)

class ProcSample():
  def __init__(self,args):
    self.args=args

  def Process(self,idx,data):
    #print idx
    #time.sleep(5)
    return data.sum()

  def Close(self):
    pass

class ClampOutSample(ClampOut):
  def __init__(self,args,job=(0,1),verbose=0):
    print 'ClampOutSample',args,job
    self.fh=open('/tmp/ProcSample-%d.log'%os.getpid(),'w')

  @staticmethod
  def GetCaps(param): return 2

  def SetData(self,idx,res):
    self.fh.write('idx %0.2d res %d\n'%(idx,res))
    print 'SetData',idx,res;sys.stdout.flush()
  def Close(self):
    print 'Close PID %d'%os.getpid()
    self.fh.close()

def RunWithArgs(args):
  if type(args)==dict:
    args=ut.dotdict(args)
    default={
            'numFrm':sys.maxint,
            'matRoi':None,
            'dst':None,
            'multiproc':None,
            'cmpr':None,
            'verbose':3
            } #setup default values
    for k, v in default.items():
      if not args.has_key(k):
        args[k]=v
    if args.multiproc==True: args.multiproc=[3]
    _log.setLevel(logging.INFO if args.verbose&1 else logging.WARNING)
  else:
    args=ut.dotdict(args.__dict__)
  for clp in (args.src,args.dst):
    if clp[0] in ('zmq','zmqMsg'):
      clp[1]=ut.ZMQ.lst2dict(clp[1:])
      del clp[2:]

  if args.multiproc!=None:
    MultiProc(ClampIn,ProcSample,ClampOutSample,args)
  else:
    SingleProc(ClampIn,ProcSample,ClampOutSample,args)

if __name__ == '__main__':
  def LogConfig(args):
    logging.basicConfig()
    hdlr=logging.StreamHandler()
    fmt = logging.Formatter('%(levelname)s:%(name)s:%(lineno)d:%(funcName)s:%(message)s')
    hdlr.setFormatter(fmt)
    del logging.root.handlers[:]
    logging.root.addHandler(hdlr)
    logging.root.setLevel(logging.DEBUG)


  def ParseArgs():
    import argparse #since python 2.7
    (h,t)=os.path.split(sys.argv[0]);cmd='\n  '+(t if len(h)>3 else sys.argv[0])+' '
    exampleCmd=(
                '--src rand uint16 2560 2160 3',
                '--src inc uint16 2560 2160 6',
                '--src raw uint16 2560 2160 /scratch/detectorData/PCO/ -v',
                '--src tif /scratch/detectorData/TestTIFF/ -v',
                '--src cbf /scratch/detectorData/e14472/pilatus_1/S00000-00999/S00033/ -v',
                '--src hdf /scratch/detectorData/e14472/scan_00033.hdf5 /entry/data/pilatus_1 -v',
                '--src zmq addr:tcp://localhost:8080 qs:4 if:PULL',
                )
    epilog='''

  To test a vairety of processing configuration try:
    for i in 0 1 2 3; do rm /tmp/ProcSample-*.log;./procSample.py --src inc uint16 2560 2160 30 -m$i,4; done
  or
    for i in 0 1 2 3; do rm /tmp/ProcSample-*.log;./procSample.py --src inc uint16 2560 2160 30 -m$i,4 -v; done

the verbose bits are:
   bit0(0x01):
   bit1(0x02):
   bit2(0x04):
   bit3(0x08):
   bit4(0x10):
   bit5(0x20):
   bit6(0x40):
   bit6(0x80):
   bit6(0x100): multiprocessing
   default is 1+2=3 (no -v)
   -v is 1+4+8+16=511
   -v0 is turn all off

multiproc-topo indicates which multiproc topologies (if possible) will be setup::

   0: 1-1  (worst)
   1: n-1
   2: 1-n
   3: n-n  (best)
  0                    1                2                 3
        --CP--         s---CP--               --CP---d    s---CP---d
       /      \\                \\             /
  s---MP--CP--MP--d    s---CP--MP--d    s---MP--CP---d    s---CP---d
       \\      /                /             \\
        --CP--         s---CP--               --CP---d    s---CP---d

The finally chosen topoligy is the best of these and the capabilities of the input and output clamps.

The involved classes are::

  +-------+   +----------+   +--------------+
  |ClampIn+->-+ProcSample+->-+ClampOutSample|
  +-------+   +----------+   +--------------+

- ClampIn is any kind of supported data source with parameters src
- ProcSample just calculates the sum of the data
- ClampOutSample just prints the value to the console and to the file: /tmp/ProcSample-%d.log


  Here the Source description for the ClampIn:'''+ClampIn.__types__+'\nExamples:'+''.join(map(lambda s:cmd+s,exampleCmd))+'\n '

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__, epilog=epilog)
    parser.add_argument('--src',    nargs='+', required=True, help='data source (see below for examples)')
    parser.add_argument('--dst',  nargs='+', metavar=('mode','param'), default=[None,], help='data destination: hdf or zmq sink')
    parser.add_argument('-v',   dest='verbose',   type=int, nargs='?', const=511, default=3, help='verbose (optional level bitmask)')
    parser.add_argument('-m', '--multiproc', nargs='?', const=[3], default=None,help='use multiprocessing method topo,[numproc]')

    args = parser.parse_args()
    if type(args.multiproc)==str:
      args.multiproc=map(int,args.multiproc.split(','))
      assert(len(args.multiproc)<=2)
    LogConfig(args)
    RunWithArgs(args)
#------------------ Main Code ----------------------------------
  ParseArgs()

