#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
ZeroMQ Push/Pull Server:
reads all *.raw files from a directory and sends this raw data as ZMQ messages.
'''

import  sys, os, time, glob, re
import zmq
from Clamp import ClampIn,ClampOut,ClampLst
import utilities as ut

class FileSrc:
  def GetData(self,idx=None):
    '''reads all files in a directory and stores the data in the imgList'''
    fnLst=self.fnLst
    if idx!=None:
      self._idx=idx
    else:
      idx=self._idx
    if idx>=len(fnLst):
      return None
    fn=fnLst[idx]
    fd=open(fn)
    arr=fd.read()
    if self._verbose:
      print idx,len(arr),fnLst[idx]
    self._idx+=1
    return (fn,arr)

  def __init__(self,path,verbose=False):
    self._verbose=verbose
    self._idx=0
    self.fnLst=ClampIn.GetGlob(path,'cbf')

class ZmqImageSource:
  ifDict={'PUSH':zmq.PUSH,'PUB':zmq.PUB}
  def Run(self,args):
    pipe=args.get('pipe')
    sock=ut.ZMQ.Socket(args,args.ifType,args.queueSz)
    #header = {'endianess': 'little', 'tag': '', 'source': '', 'version': 1}
    header = {}
    print 'started ZmqImageSource on address %s serving %s files...'%(args.addr,args.src)

    if args.src[0]=='raw':
      fnLst=[]
      glb=args.src[1]+'/*'
      glb=os.path.expanduser(glb)
      #fnLst=glob.iglob(glb)
      fnLst.extend(glob.iglob(glb))
      fnLst.sort()
      msgId=int(re.match('(\d*)',os.path.basename(fnLst[0])).group(0))
      for i in range(len(fnLst)):
        fn=fnLst[i]
        try:
          msgIdNxt=int(re.match('(\d*)',os.path.basename(fnLst[i+1])).group(0))
        except IndexError:
          msgId=-1
        fd=open(fn)
        arr=fd.read()
        if args.verbose:
          print fn
        if msgId==msgIdNxt:
          sock.send(arr, zmq.SNDMORE)#file content
        else:
          sock.send(arr)#file content
          msgId=msgIdNxt
        fd.close()
      pass
    elif args.src[0]=='file':
      idx=0
      fs=FileSrc(args.src[1],args.verbose)
      while True:
        if pipe and pipe.poll():
          cmd=pipe.recv()
          if cmd=='stop':
            break
          elif cmd=='frmCnt':
            pipe.send(str(idx))
        #print sock.SNDBUF,sock.SNDHWM,sock.RCVBUF,sock.RCVHWM
        if args.numFrm and args.numFrm==idx:
          break
        data=fs.GetData()
        if data==None:
          if args.infinite or args.numFrm :
            data=fs.GetData(0)#reset to start
          else:
            break
        header['frame']=idx
        header['timestamp']=time.ctime()
        header[u'htype']=u'pilatus-1.0'
        (path,fnBase)=os.path.split(data[0])
        header[u'filename']=fnBase.decode('utf-8')
        header['srcpath']=path.decode('utf-8')#source path
        try:
          header[u'path']=args.src[2].decode('utf-8')#dst path
        except IndexError:
          pass
        if args.verbose:
          print idx,header
        else:
          sys.stdout.write('.');sys.stdout.flush()
        #Send multipart message
        sock.send_json(header, zmq.SNDMORE) #@UndefinedVariable
        sock.send(data[1])#file content
        #time.sleep(.2)
        idx+=1
      pass
    else:
      clpIn=ClampIn(args)
      cmpr=args.get('cmpr') #if the zmq-chunk is compressed, use it
      if cmpr:
        ChunkBuild=ut.GetChunkBuildFunc(cmpr)
      idx=0
      while True:
        if pipe and pipe.poll():
          cmd=pipe.recv()
          if cmd=='stop':
            break
          elif cmd=='frmCnt':
            pipe.send(str(idx))
        #print sock.SNDBUF,sock.SNDHWM,sock.RCVBUF,sock.RCVHWM
        if args.numFrm and args.numFrm==idx:
          break
        arr=clpIn.GetData()
        if arr==None:
          if args.infinite:
            clpIn.SetIdx(0)#reset to start
            arr=clpIn.GetData()
          else:
            break
        #arr[arr==4294967295]=0
        header['frame']=idx
        header["timestamp"]=time.ctime()
        header['htype']='chunk-1.0'
        #header['shape']=tuple(reversed(arr.shape))
        header['shape']=tuple(reversed(arr.shape))
        header['type'] =str(arr.dtype)#'uint16'
        if args.tag:
          header['tag']=args.tag
        if cmpr:
          (cmpArr,filter)=ChunkBuild(arr)
          if filter:
            header.pop('cmpr')
          else:
            header['cmpr']=cmpr
            arr=cmpArr
        if args.verbose:
          print idx,header
        else:
          sys.stdout.write('.');sys.stdout.flush()
        try:
          #Send multipart message
          sock.send_json(header, zmq.SNDMORE) #@UndefinedVariable
          sock.send(arr)
          #time.sleep(1)
        except zmq.error.Again as e:
          print e
        idx+=1
    if not args.verbose:
      sys.stdout.write('\n');sys.stdout.flush()
    if args.stopCmd:
      sock.send_json({'cmd':'stop'})
    if pipe: pipe.send(str(idx)) #send the frame numbers to the pipe
    print 'closing...'
    #sometimes a message is lost.
    #This seems to happen when multiple consumers read from a queue
    #In this case even the LINGER option does not help.
    #the for now only working option is to wait 1 second before closing the socket.
    #I know... it is really ugly...
    print 'wait 1 sec before closing socket...';time.sleep(1)
    sock.LINGER=1000
    sock.close()
    print 'closed.'

def RunWithArgs(args):
  if type(args)==dict:
    args=ut.dotdict(args)
    #default values
    default={
            'addr':'tcp://*:8080',
            'cmpr':None,
            'ifType':'PUSH',
            'queueSz':4,
            'tag':None,
            'numFrm':sys.maxint,
            'verbose':False,
            'stopCmd':False,
            'infinite':False,
            } #setup default values
    for k, v in default.items():
      if not args.has_key(k):
        args[k]=v
  else:
    args=ut.dotdict(args.__dict__)
  if type(args.ifType)==str:
    args.ifType=ZmqImageSource.ifDict[args.ifType]
  try:
    ZmqImageSource().Run(args)
  except KeyboardInterrupt:
    print '^C received, stop zmq imageSource connection'
  print 'done.'

if __name__ == '__main__':
  import argparse #since python 2.7
  def ParseArgs():
    ifDict={'PUSH':zmq.PUSH,'PUB':zmq.PUB}
    #default values
    ifType='PUSH'
    queueSz=4

    addr='tcp://*:8080'
    (h,t)=os.path.split(sys.argv[0]);cmd='\n  '+(t if len(h)>3 else sys.argv[0])+' '
    exampleCmd=('--src raw uint16 2560 2160 /scratch/detectorData/PCO/',
                '--src rand uint16 2560 2160 3',
                '--src inc uint16 2560 2160 6',
                '--src inc uint16 2560 2160 6 -s',
                '-src raw uint16 2560 2160 /scratch/detectorData/PCO/ -v -s -i --addr=tcp://*:8080 -v',
                '-v -s -i --addr=tcp://*:8080 --src cbf /scratch/detectorData/stxm_pinhole/pilatus_1/S00000-00999/S00150/ /tmp/out',
                '-v -s -i --addr=tcp://*:8080 --src file /scratch/detectorData/stxm_pinhole/pilatus_1/S00000-00999/S00150/ /tmp/out',
                )


    epilog='Data Source description: (htype=chunk)'+ClampIn.__types__+\
    '''Data Source description: (htype=pilatus)\n  Files from a given directory:
    --src file PATH_or_PATTERN DSTPATH
    --src file /scratch/detectorData/e14472/pilatus_1/*.cbf
    --src file
    --src file /scratch/detectorData/e14472/pilatus_1/

Examples:'''+''.join(map(lambda s:cmd+s,exampleCmd))+'\n '

    #for e in exampleCmd:
    #  epilog+=' '+os.path.basename(sys.argv[0])+' '+e+'\n'

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog=epilog)
    parser.add_argument('--src',    nargs='+', required=True, help='data source (see below for examples)')
    parser.add_argument('--addr', nargs='?', default=addr, help='host:port of the zmq connection (default:%s)'%addr)
    parser.add_argument('--cmpr',  help='prefix ''s-'' for byte shuffle, compression zlib[0-9],lz4,lzf,cbf. e.g. s-zlib5: shuffle zlib level 5 compression.')
    parser.add_argument('--tag',   help='tag of the source. An identifier of the source')
    parser.add_argument('-v', dest='verbose', action='store_true',    help='verbose')
    parser.add_argument('-s', dest='stopCmd', action='store_true',    help='send stop command')
    parser.add_argument('-i', dest='infinite', action='store_true',    help='infinite loop')
    parser.add_argument('--numFrames', dest='numFrm',  type=int, help='maximal number of frames to receive')
    parser.add_argument('--if',  dest='ifType', choices=ifDict,default=ifType, help='interface type (default:'+ifType+')')
    parser.add_argument('--qs',  dest='queueSz', type=int,default=queueSz, help='queue size (default:'+str(queueSz)+')')

    args = parser.parse_args()
    RunWithArgs(args)

#------------------ Main Code ----------------------------------
  ParseArgs()
