#!/usr/bin/env python

# *-----------------------------------------------------------------------*
# |                                                                       |
# |  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
# |                                                                       |
# |              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
# *-----------------------------------------------------------------------*
'''
simple movie example of a hdf5 file.
'''

import os,sys,h5py
import numpy as np
import matplotlib.pyplot as plt

class Hdf5vis:
  def __init__(self,fnHDF):
    self.Open(fnHDF)

  def __del__(self):
    self.Close()

  def Open(self,fnHDF):
    self.fid = h5py.h5f.open(fnHDF)

  def Close(self):
    self.fid.close()
    del self.fid
    #print 'closed'

  @staticmethod
  def _ShowHirarchy(gidParent,lvl):
    for gidStr in h5py.h5g.GroupIter(gidParent):
      gid = h5py.h5o.open(gidParent,gidStr)
      print lvl*' '+gidStr,type(gid)
      if type(gid)==h5py.h5g.GroupID:
        Hdf5vis._ShowHirarchy(gid,lvl+1)

  def ShowHirarchy(self):
    Hdf5vis._ShowHirarchy(self.fid,0)


  def Movie(self,path,color):

    dsid=h5py.h5d.open(self.fid,path)
    ds=h5py.Dataset(dsid)
    shape=ds.shape
    print shape
    for i in range(shape[0]):
      #print i
      frame=ds[i,...]
      titstr=path+' %d'%i
      print frame
      try:
        title.set_text(titstr)
        ih.set_array(frame)
        #ih._extent=None
        #ih.set_extent(ih.get_extent())
      except UnboundLocalError as e:
        plt.ion()
        f=plt.figure();title=f.suptitle(titstr)
        if color==None:
          ih=plt.imshow(frame,interpolation='nearest')
        elif color[0]=='avgStd':
          avg=np.average(frame);std=np.std(frame);print avg,std
          ih=plt.imshow(frame,vmin=max(0,avg-3*std), vmax=avg+3*std,interpolation='nearest')
        else:
          vmin=eval(color[0]);vmax=eval(color[1])
          ih=plt.imshow(frame,vmin=vmin,vmax=vmax,interpolation='nearest')
      plt.draw()

if __name__ == '__main__':
  def RunWithArgs():
    import argparse #since python 2.7
    exampleCmd="/scratch/detectorData/e14472_00033.hdf5 -H -m=entry/dataScan00033/pilatus_2"
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('file', help='hdf5 file to read')
    #parser.add_argument('--eAccount', help='the used eAccount name')
    parser.add_argument('-m', '--movie',  help='make movie of this path')
    parser.add_argument('-H', '--hirarchy', action='store_true', help='show hirarchy of file')
    parser.add_argument('-c', '--color', nargs='+', help='color mapping min, max, standard dev')

    args = parser.parse_args()

    hdf5vis=Hdf5vis(args.file)
    if args.hirarchy:
      hdf5vis.ShowHirarchy()
    if args.movie:
      hdf5vis.Movie(args.movie,args.color)
#---------- MAIN TEST CODE ----------
  RunWithArgs()
