#!/usr/bin/python

import sys
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import hsv_to_rgb
import h5py

if len(sys.argv) != 2:
    print 'Use: %s <solutions file>' % sys.argv[0]
    sys.exit(-1)

f = h5py.File(sys.argv[1], 'r')

g = f['probes']
prb = []
for p in g:
    print 'Reading', p, '...'
    prb_t = np.array(g[p])
    prb_s = prb_t.shape
    if len(prb_s) == 2:
        prb_t.resize(1, prb_s[0], prb_s[1])
    prb.append(prb_t)

g = f['objects']
obj = []
for o in g:
    print 'Reading', o, '...'
    obj_t = np.array(g[o])
    obj_s = obj_t.shape
    if len(obj_s) == 2:
        obj_t.resize(1, obj_s[0], obj_s[1])
    obj.append(obj_t)

f.close()

figsz = (8, 8)
fig = plt.figure(figsize=figsz)
ax = plt.gca()
ax.xaxis.set_major_locator(plt.NullLocator())
ax.yaxis.set_major_locator(plt.NullLocator())

for p in xrange(len(prb)):
    s = prb[p].shape
    for m in xrange(s[0]):
        prb_p = prb[p][m,:,:]

        prb_phase = (np.angle(prb_p) + np.pi) / (2*np.pi)
        prb_amp = np.absolute(prb_p)
        prb_amp /= prb_amp.max()

        pnr, pnc = prb_p.shape
        prb_hsv = np.ndarray((pnr, pnc, 3))
        prb_hsv[:,:,0] = prb_phase
        prb_hsv[:,:,1] = 1.0
        prb_hsv[:,:,2] = prb_amp

        print 'Show probe %d/%d'%(p, m)
        plt.title('Probe '+str(p)+'/'+str(m))
        plt.imshow(hsv_to_rgb(prb_hsv))
        plt.show()
        plt.clf()

for o in xrange(len(obj)):
    s = obj[o].shape
    for m in xrange(s[0]):
        obj_o = obj[o][m,:,:]

        print 'Show object %d/%d'%(o, m)
        onr, onc = obj_o.shape
        obj_phase = (np.angle(obj_o) + np.pi) / (2*np.pi)

        print '  displaying object phase'
        dr = pnr/2
        dc = pnc/2
        plt.title('Object '+str(o)+'/'+str(m)+' phase')
        plt.imshow(obj_phase, cmap=plt.cm.bone)
        oprr = obj_phase[dr:onr-dr,dc:onc-dc]
        plt.clim(np.min(oprr), np.max(oprr))
        plt.show()
        plt.clf()

        obj_amp = np.absolute(obj_o)
        print '  max amplitude is', obj_amp.max()
        obj_amp /= obj_amp.max()

        print '  displaying object amplitude'
        plt.title('Object '+str(o)+'/'+str(m)+' amplitude')
        plt.imshow(obj_amp, cmap=plt.cm.bone)
        oprr = obj_amp[dr:onr-dr,dc:onc-dc]
        plt.clim(np.min(oprr), np.max(oprr))
        plt.show()
        plt.clf()
