#!/bin/bash
#  +-----------+                 +-----------+
#  |           |   8080          |           |
#  | zmqImgSrc |+--------------->| zmqWriter |
#  |           |                 |           |
#  +-----------+                 +-----------+
#
# 1.  uint32                   +
# 2.  uint16                   |
# 3.  uint8                    |
# 3.  uint8 n4                 + -> write to hdf5 with: no cmpr, xxx-zlib4, xxx-zlib4 multiproc
#
# 4.  2 sources streamed to one hdf5 file
source configTest.sh

runit()
{
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriter32$PF.hdf5 entry/data/dataset --timeout 10000 --numFrames=100 $ARGS $CMP0
WaitAll

$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint16 256 216 100 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriter16$PF.hdf5 entry/data/dataset --timeout 10000 --numFrames=100 $ARGS $CMP0
WaitAll

$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint8 256 216 100 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriter8$PF.hdf5 entry/data/dataset --timeout 10000 --numFrames=100 $ARGS $CMP1
WaitAll

$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint8 256 216 100 --cmpr n4 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriter4$PF.hdf5 entry/data/dataset --timeout 10000 --numFrames=100 $ARGS $CMP2
WaitAll
}

RunTest()
{
case "$1" in
0) #0 ?
  case "$2" in
  0) #0 0
  set -x
  ARGS=
  CMP0=
  CMP1=
  CMP2=
  PF=
  ;;
  1) #0 1
  ARGS=
  CMP0='--cmpr s-zlib4'
  CMP1='--cmpr zlib4'
  CMP2='--cmpr n4-zlib4'
  PF=cmp
  ;;
  2) #0 2
  ARGS='--multiproc'
  CMP0='--cmpr s-zlib4'
  CMP1='--cmpr zlib4'
  CMP2='--cmpr n4-zlib4'
  PF=cmpmp
  ;;
  esac
  runit
  ;;
1) #1
#BrQ and AqE are the relative path of the dataset under the root entry
#all steams have the same compression
$XT0 "$LXR/zmqImageSource.py --addr=tcp://localhost:8080 --src inc uint16 640 480 100 --tag BrQ -v;$ENDCMD"&
$XT1 "$LXR/zmqImageSource.py --addr=tcp://localhost:8080 --src inc uint32 256 216 100 --tag AqE -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://*:8080 --dst hdf $OUTDIR/zmqWriter2Stream1.hdf5 entry/data/dataset --timeout 10000 --numFrames=200 --cmpr s-zlib4
WaitAll
#or with tag2path conversion and multiprocessing
$XT0 "$LXR/zmqImageSource.py --addr=tcp://localhost:8080 --src inc uint16 640 480 100 --tag BrQ -v;$ENDCMD"&
$XT1 "$LXR/zmqImageSource.py --addr=tcp://localhost:8080 --src inc uint32 256 216 100 --tag AqE -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://*:8080 --dst hdf $OUTDIR/zmqWriter2Stream2.hdf5 entry/data/dataset --timeout 10000 --numFrames=200 --cmpr s-zlib4 --multiproc --tag2path '{"AqE":"the/Path/OF_AqE"}'
WaitAll
;;
2) #2
#ofsDim to change indexing and to make multidim Arrays
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint16 8 16 32 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriterOfsDim1.hdf5 entry/data/dataset --timeout 10000 --numFrames=32 --ofsDim '[0, 1, 5]'
WaitAll
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src rand uint16 8 16 32 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriterOfsDim2.hdf5 entry/data/dataset a --timeout 10000 --numFrames=32 --ofsDim '[0, 3, 5]'
WaitAll
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src rand uint16 8 16 22 -v;$ENDCMD"&
$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriterOfsDim3.hdf5 entry/data/dataset a --timeout 10000 --numFrames=22 --ofsDim '[10, 4, 5]'
WaitAll
;;
esac
}

RunTest $1 $2
#ls -l $OUTDIR/zmqWriter*



